@@ -3,6 +3,7 @@
 use std::cell::Cell;
 use std::ffi::c_void;
 use std::ops::Deref;
+use std::ptr;
 use std::ptr::null_mut;
 use std::ptr::NonNull;
 use std::slice;
@@ -233,11 +234,12 @@ pub type BackingStoreDeleterCallback = unsafe extern "C" fn(
 
 pub unsafe extern "C" fn backing_store_deleter_callback(
   data: *mut c_void,
-  _byte_length: usize,
+  byte_length: usize,
   _deleter_data: *mut c_void,
 ) {
-  let b = Box::from_raw(data);
-  drop(b)
+  let slice_ptr = ptr::slice_from_raw_parts_mut(data as *mut u8, byte_length);
+  let b = Box::from_raw(slice_ptr);
+  drop(b);
 }
 
 /// A wrapper around the backing store (i.e. the raw memory) of an array buffer.
