@@ -99,6 +99,11 @@ extern "C" {
     context: *const Context,
     index: u32,
   ) -> MaybeBool;
+  fn v8__Object__HasOwnProperty(
+    this: *const Object,
+    context: *const Context,
+    key: *const Name,
+  ) -> MaybeBool;
   fn v8__Object__Delete(
     this: *const Object,
     context: *const Context,
@@ -446,6 +451,18 @@ impl Object {
       .into()
   }
 
+  /// HasOwnProperty() is like JavaScript's Object.prototype.hasOwnProperty().
+  pub fn has_own_property<'s>(
+    &self,
+    scope: &mut HandleScope<'s>,
+    key: Local<Name>,
+  ) -> Option<bool> {
+    unsafe {
+      v8__Object__HasOwnProperty(self, &*scope.get_current_context(), &*key)
+    }
+    .into()
+  }
+
   pub fn delete<'s>(
     &self,
     scope: &mut HandleScope<'s>,
