@@ -17,9 +17,7 @@
 // and resetting these counters.
 
 // Make sure to sync the following with src/compiler/globals.h.
-#if defined(V8_TARGET_ARCH_X64) || defined(V8_TARGET_ARCH_ARM64) || \
-    (defined(V8_TARGET_ARCH_MIPS64) && !defined(USE_SIMULATOR)) ||  \
-    (defined(V8_TARGET_ARCH_LOONG64) && !defined(USE_SIMULATOR))
+#if defined(V8_TARGET_ARCH_X64) || defined(V8_TARGET_ARCH_ARM64)
 #define V8_ENABLE_FP_PARAMS_IN_C_LINKAGE
 #endif
 
