@@ -22,6 +22,7 @@
 #include "src/handles/global-handles.h"
 #include "src/handles/handles.h"
 #include "src/heap/base/stack.h"
+#include "src/heap/cppgc-js/cpp-marking-state.h"
 #include "src/heap/cppgc-js/cpp-snapshot.h"
 #include "src/heap/cppgc-js/unified-heap-marking-state.h"
 #include "src/heap/cppgc-js/unified-heap-marking-verifier.h"
@@ -91,12 +92,6 @@ void CppHeap::CollectGarbageForTesting(cppgc::EmbedderStackState stack_state) {
   return internal::CppHeap::From(this)->CollectGarbageForTesting(stack_state);
 }
 
-void JSHeapConsistency::DijkstraMarkingBarrierSlow(
-    cppgc::HeapHandle& heap_handle, const TracedReferenceBase& ref) {
-  auto& heap_base = cppgc::internal::HeapBase::From(heap_handle);
-  static_cast<JSVisitor*>(&heap_base.marker()->Visitor())->Trace(ref);
-}
-
 void JSHeapConsistency::CheckWrapper(v8::Local<v8::Object>& wrapper,
                                      int wrapper_index, const void* wrappable) {
   CHECK_EQ(wrappable,
@@ -176,15 +171,31 @@ UnifiedHeapConcurrentMarker::CreateConcurrentMarkingVisitor(
       heap(), marking_state, unified_heap_marking_state_);
 }
 
+void FatalOutOfMemoryHandlerImpl(const std::string& reason,
+                                 const SourceLocation&, HeapBase* heap) {
+  FatalProcessOutOfMemory(static_cast<v8::internal::CppHeap*>(heap)->isolate(),
+                          reason.c_str());
+}
+
+}  // namespace
+
 class UnifiedHeapMarker final : public cppgc::internal::MarkerBase {
  public:
-  UnifiedHeapMarker(Key, Heap* v8_heap, cppgc::internal::HeapBase& cpp_heap,
+  UnifiedHeapMarker(Heap* v8_heap, cppgc::internal::HeapBase& cpp_heap,
                     cppgc::Platform* platform, MarkingConfig config);
 
   ~UnifiedHeapMarker() final = default;
 
   void AddObject(void*);
 
+  cppgc::internal::MarkingWorklists& GetMarkingWorklists() {
+    return marking_worklists_;
+  }
+
+  cppgc::internal::MarkingStateBase& GetMutatorMarkingState() {
+    return marking_visitor_.marking_state_;
+  }
+
  protected:
   cppgc::Visitor& visitor() final { return marking_visitor_; }
   cppgc::internal::ConservativeTracingVisitor& conservative_visitor() final {
@@ -200,11 +211,11 @@ class UnifiedHeapMarker final : public cppgc::internal::MarkerBase {
   cppgc::internal::ConservativeMarkingVisitor conservative_marking_visitor_;
 };
 
-UnifiedHeapMarker::UnifiedHeapMarker(Key key, Heap* v8_heap,
+UnifiedHeapMarker::UnifiedHeapMarker(Heap* v8_heap,
                                      cppgc::internal::HeapBase& heap,
                                      cppgc::Platform* platform,
                                      MarkingConfig config)
-    : cppgc::internal::MarkerBase(key, heap, platform, config),
+    : cppgc::internal::MarkerBase(heap, platform, config),
       unified_heap_marking_state_(v8_heap),
       marking_visitor_(heap, mutator_marking_state_,
                        unified_heap_marking_state_),
@@ -220,16 +231,9 @@ void UnifiedHeapMarker::AddObject(void* object) {
       cppgc::internal::HeapObjectHeader::FromObject(object));
 }
 
-void FatalOutOfMemoryHandlerImpl(const std::string& reason,
-                                 const SourceLocation&, HeapBase* heap) {
-  FatalProcessOutOfMemory(static_cast<v8::internal::CppHeap*>(heap)->isolate(),
-                          reason.c_str());
-}
-
-}  // namespace
-
 void CppHeap::MetricRecorderAdapter::AddMainThreadEvent(
     const FullCycle& cppgc_event) {
+  DCHECK(!last_full_gc_event_.has_value());
   last_full_gc_event_ = cppgc_event;
   GetIsolate()->heap()->tracer()->NotifyGCCompleted();
 }
@@ -300,12 +304,23 @@ bool CppHeap::MetricRecorderAdapter::MetricsReportPending() const {
 
 const base::Optional<cppgc::internal::MetricRecorder::FullCycle>
 CppHeap::MetricRecorderAdapter::ExtractLastFullGcEvent() {
-  return std::move(last_full_gc_event_);
+  auto res = std::move(last_full_gc_event_);
+  last_full_gc_event_.reset();
+  return res;
 }
 
 const base::Optional<cppgc::internal::MetricRecorder::MainThreadIncrementalMark>
 CppHeap::MetricRecorderAdapter::ExtractLastIncrementalMarkEvent() {
-  return std::move(last_incremental_mark_event_);
+  auto res = std::move(last_incremental_mark_event_);
+  last_incremental_mark_event_.reset();
+  return res;
+}
+
+void CppHeap::MetricRecorderAdapter::ClearCachedEvents() {
+  incremental_mark_batched_events_.events.clear();
+  incremental_sweep_batched_events_.events.clear();
+  last_incremental_mark_event_.reset();
+  last_full_gc_event_.reset();
 }
 
 Isolate* CppHeap::MetricRecorderAdapter::GetIsolate() const {
@@ -397,17 +412,6 @@ void CppHeap::DetachIsolate() {
   no_gc_scope_++;
 }
 
-void CppHeap::RegisterV8References(
-    const std::vector<std::pair<void*, void*> >& embedder_fields) {
-  DCHECK(marker_);
-  for (auto& tuple : embedder_fields) {
-    // First field points to type.
-    // Second field points to object.
-    static_cast<UnifiedHeapMarker*>(marker_.get())->AddObject(tuple.second);
-  }
-  marking_done_ = false;
-}
-
 namespace {
 
 bool IsMemoryReducingGC(CppHeap::GarbageCollectionFlags flags) {
@@ -423,9 +427,14 @@ bool ShouldReduceMemory(CppHeap::GarbageCollectionFlags flags) {
 }
 
 }  // namespace
-void CppHeap::TracePrologue(GarbageCollectionFlags gc_flags) {
+
+void CppHeap::InitializeTracing(GarbageCollectionFlags gc_flags) {
   CHECK(!sweeper_.IsSweepingInProgress());
 
+  // Check that previous cycle metrics have been reported.
+  DCHECK_IMPLIES(GetMetricRecorder(),
+                 !GetMetricRecorder()->MetricsReportPending());
+
 #if defined(CPPGC_YOUNG_GENERATION)
   cppgc::internal::SequentialUnmarker unmarker(raw_heap());
 #endif  // defined(CPPGC_YOUNG_GENERATION)
@@ -451,10 +460,13 @@ void CppHeap::TracePrologue(GarbageCollectionFlags gc_flags) {
     compactor_.InitializeIfShouldCompact(marking_config.marking_type,
                                          marking_config.stack_state);
   }
-  marker_ =
-      cppgc::internal::MarkerFactory::CreateAndStartMarking<UnifiedHeapMarker>(
-          isolate_ ? isolate_->heap() : nullptr, AsBase(), platform_.get(),
-          marking_config);
+  marker_ = std::make_unique<UnifiedHeapMarker>(
+      isolate_ ? isolate()->heap() : nullptr, AsBase(), platform_.get(),
+      marking_config);
+}
+
+void CppHeap::StartTracing() {
+  marker_->StartMarking();
   marking_done_ = false;
 }
 
@@ -599,7 +611,10 @@ void CppHeap::CollectGarbageForTesting(
   } else {
     // Perform an atomic GC, with starting incremental/concurrent marking and
     // immediately finalizing the garbage collection.
-    if (!IsMarking()) TracePrologue(GarbageCollectionFlagValues::kForced);
+    if (!IsMarking()) {
+      InitializeTracing(GarbageCollectionFlagValues::kForced);
+      StartTracing();
+    }
     EnterFinalPause(stack_state);
     AdvanceTracing(std::numeric_limits<double>::infinity());
     TraceEpilogue();
@@ -620,7 +635,8 @@ void CppHeap::StartIncrementalGarbageCollectionForTesting() {
   DCHECK_NULL(isolate_);
   if (IsMarking()) return;
   force_incremental_marking_for_testing_ = true;
-  TracePrologue(GarbageCollectionFlagValues::kForced);
+  InitializeTracing(GarbageCollectionFlagValues::kForced);
+  StartTracing();
   force_incremental_marking_for_testing_ = false;
 }
 
@@ -719,5 +735,22 @@ CppHeap::MetricRecorderAdapter* CppHeap::GetMetricRecorder() const {
 
 void CppHeap::FinishSweepingIfRunning() { sweeper_.FinishIfRunning(); }
 
+std::unique_ptr<CppMarkingState> CppHeap::CreateCppMarkingState() {
+  DCHECK(IsMarking());
+  return std::make_unique<CppMarkingState>(
+      isolate(), wrapper_descriptor_,
+      std::make_unique<cppgc::internal::MarkingStateBase>(
+          AsBase(),
+          static_cast<UnifiedHeapMarker*>(marker())->GetMarkingWorklists()));
+}
+
+std::unique_ptr<CppMarkingState>
+CppHeap::CreateCppMarkingStateForMutatorThread() {
+  DCHECK(IsMarking());
+  return std::make_unique<CppMarkingState>(
+      isolate(), wrapper_descriptor_,
+      static_cast<UnifiedHeapMarker*>(marker())->GetMutatorMarkingState());
+}
+
 }  // namespace internal
 }  // namespace v8
