@@ -27,6 +27,7 @@ class StackFrame;
 class V8Debugger;
 class V8DebuggerAgentImpl;
 class V8InspectorImpl;
+class V8RuntimeAgentImpl;
 class V8StackTraceImpl;
 struct V8StackTraceId;
 
@@ -85,6 +86,9 @@ class V8Debugger : public v8::debug::DebugDelegate,
   int maxAsyncCallChainDepth() { return m_maxAsyncCallStackDepth; }
   void setAsyncCallStackDepth(V8DebuggerAgentImpl*, int);
 
+  int maxCallStackSizeToCapture() const { return m_maxCallStackSizeToCapture; }
+  void setMaxCallStackSizeToCapture(V8RuntimeAgentImpl*, int);
+
   std::shared_ptr<AsyncStackTrace> currentAsyncParent();
   V8StackTraceId currentExternalParent();
 
@@ -248,6 +252,7 @@ class V8Debugger : public v8::debug::DebugDelegate,
 
   size_t m_maxAsyncCallStacks;
   int m_maxAsyncCallStackDepth;
+  int m_maxCallStackSizeToCapture;
 
   std::vector<void*> m_currentTasks;
   std::vector<std::shared_ptr<AsyncStackTrace>> m_currentAsyncParent;
@@ -259,6 +264,7 @@ class V8Debugger : public v8::debug::DebugDelegate,
   std::list<std::shared_ptr<AsyncStackTrace>> m_allAsyncStacks;
 
   std::unordered_map<V8DebuggerAgentImpl*, int> m_maxAsyncCallStackDepthMap;
+  std::unordered_map<V8RuntimeAgentImpl*, int> m_maxCallStackSizeToCaptureMap;
   void* m_taskWithScheduledBreak = nullptr;
 
   // If any of the following three is true, we schedule pause on next JS
