@@ -361,55 +361,73 @@ class InternalizedStringKey final : public StringTableKey {
     return string_->SlowEquals(string);
   }
 
-  Handle<String> AsHandle(Isolate* isolate) {
-    // Internalize the string in-place if possible.
-    MaybeHandle<Map> maybe_internalized_map;
+  void PrepareForInsertion(Isolate* isolate) {
     StringTransitionStrategy strategy =
         isolate->factory()->ComputeInternalizationStrategyForString(
-            string_, &maybe_internalized_map);
+            string_, &maybe_internalized_map_);
     switch (strategy) {
       case StringTransitionStrategy::kCopy:
         break;
       case StringTransitionStrategy::kInPlace:
-        // A relaxed write is sufficient here even with concurrent
-        // internalization. Though it is not synchronizing, a thread that does
-        // not see the relaxed write will wait on the string table write
-        // mutex. When that thread acquires that mutex, the ordering of the
-        // mutex's underlying memory access will force this map update to become
-        // visible to it.
-        string_->set_map_no_write_barrier(
-            *maybe_internalized_map.ToHandleChecked());
-        DCHECK(string_->IsInternalizedString());
-        return string_;
+        // In-place transition will be done in GetHandleForInsertion, when we
+        // are sure that we are going to insert the string into the table.
+        return;
       case StringTransitionStrategy::kAlreadyTransitioned:
         // We can see already internalized strings here only when sharing the
         // string table and allowing concurrent internalization.
         DCHECK(FLAG_shared_string_table);
-        return string_;
+        return;
     }
 
+    // Copying the string here is always threadsafe, as no instance type
+    // requiring a copy can transition any further.
+    StringShape shape(*string_);
     // External strings get special treatment, to avoid copying their
     // contents as long as they are not uncached.
-    StringShape shape(*string_);
     if (shape.IsExternalOneByte() && !shape.IsUncachedExternal()) {
       // TODO(syg): External strings not yet supported.
       DCHECK(!FLAG_shared_string_table);
-      return isolate->factory()
-          ->InternalizeExternalString<ExternalOneByteString>(string_);
+      string_ =
+          isolate->factory()->InternalizeExternalString<ExternalOneByteString>(
+              string_);
     } else if (shape.IsExternalTwoByte() && !shape.IsUncachedExternal()) {
       // TODO(syg): External strings not yet supported.
       DCHECK(!FLAG_shared_string_table);
-      return isolate->factory()
-          ->InternalizeExternalString<ExternalTwoByteString>(string_);
+      string_ =
+          isolate->factory()->InternalizeExternalString<ExternalTwoByteString>(
+              string_);
     } else {
       // Otherwise allocate a new internalized string.
-      return isolate->factory()->NewInternalizedStringImpl(
+      string_ = isolate->factory()->NewInternalizedStringImpl(
           string_, string_->length(), string_->raw_hash_field());
     }
   }
 
+  Handle<String> GetHandleForInsertion() {
+    Handle<Map> internalized_map;
+    // When preparing the string, the strategy was to in-place migrate it.
+    if (maybe_internalized_map_.ToHandle(&internalized_map)) {
+      // It is always safe to overwrite the map. The only transition possible
+      // is another thread migrated the string to internalized already.
+      // Migrations to thin are impossible, as we only call this method on table
+      // misses inside the critical section.
+      string_->set_map_no_write_barrier(*internalized_map);
+      DCHECK(string_->IsInternalizedString());
+      return string_;
+    }
+    // We prepared an internalized copy for the string or the string was already
+    // internalized.
+    // In theory we could have created a copy of a SeqString in young generation
+    // that has been promoted to old space by now. In that case we could
+    // in-place migrate the original string instead of internalizing the copy
+    // and migrating the original string to a ThinString. This scenario doesn't
+    // seem to be common enough to justify re-computing the strategy here.
+    return string_;
+  }
+
  private:
   Handle<String> string_;
+  MaybeHandle<Map> maybe_internalized_map_;
 };
 
 Handle<String> StringTable::LookupString(Isolate* isolate,
@@ -510,14 +528,7 @@ Handle<String> StringTable::LookupKey(IsolateT* isolate, StringTableKey* key) {
   }
 
   // No entry found, so adding new string.
-
-  // Allocate the string before the first insertion attempt, reuse this
-  // allocated value on insertion retries. If another thread concurrently
-  // allocates the same string, the insert will fail, the lookup above will
-  // succeed, and this string will be discarded.
-  Handle<String> new_string = key->AsHandle(isolate);
-  DCHECK_IMPLIES(FLAG_shared_string_table, new_string->IsShared());
-
+  key->PrepareForInsertion(isolate);
   {
     base::MutexGuard table_write_guard(&write_mutex_);
 
@@ -531,12 +542,16 @@ Handle<String> StringTable::LookupKey(IsolateT* isolate, StringTableKey* key) {
     if (element == empty_element()) {
       // This entry is empty, so write it and register that we added an
       // element.
+      Handle<String> new_string = key->GetHandleForInsertion();
+      DCHECK_IMPLIES(FLAG_shared_string_table, new_string->IsShared());
       data->Set(entry, *new_string);
       data->ElementAdded();
       return new_string;
     } else if (element == deleted_element()) {
       // This entry was deleted, so overwrite it and register that we
       // overwrote a deleted element.
+      Handle<String> new_string = key->GetHandleForInsertion();
+      DCHECK_IMPLIES(FLAG_shared_string_table, new_string->IsShared());
       data->Set(entry, *new_string);
       data->DeletedElementOverwritten();
       return new_string;
@@ -663,7 +678,15 @@ Address StringTable::Data::TryStringToIndexOrLookupExisting(Isolate* isolate,
   }
 
   String internalized = String::cast(string_table_data->Get(isolate, entry));
-  string.MakeThin(isolate, internalized);
+  // string can be internalized here, if another thread internalized it.
+  // If we found and entry in the string table and string is not internalized,
+  // there is no way that it can transition to internalized later on. So a last
+  // check here is sufficient.
+  if (!string.IsInternalizedString()) {
+    string.MakeThin(isolate, internalized);
+  } else {
+    DCHECK(FLAG_shared_string_table);
+  }
   return internalized.ptr();
 }
 
@@ -671,7 +694,12 @@ Address StringTable::Data::TryStringToIndexOrLookupExisting(Isolate* isolate,
 Address StringTable::TryStringToIndexOrLookupExisting(Isolate* isolate,
                                                       Address raw_string) {
   String string = String::cast(Object(raw_string));
-  DCHECK(!string.IsInternalizedString());
+  if (string.IsInternalizedString()) {
+    // string could be internalized, if the string table is shared and another
+    // thread internalized it.
+    DCHECK(FLAG_shared_string_table);
+    return raw_string;
+  }
 
   // Valid array indices are >= 0, so they cannot be mixed up with any of
   // the result sentinels, which are negative.
