@@ -45,8 +45,7 @@ Object CompileOptimized(Isolate* isolate, Handle<JSFunction> function,
   // As a post-condition of CompileOptimized, the function *must* be compiled,
   // i.e. the installed Code object must not be the CompileLazy builtin.
   DCHECK(function->is_compiled());
-  // TODO(v8:11880): avoid roundtrips between cdc and code.
-  return ToCodeT(function->code());
+  return function->code();
 }
 
 }  // namespace
@@ -76,8 +75,7 @@ RUNTIME_FUNCTION(Runtime_CompileLazy) {
     return ReadOnlyRoots(isolate).exception();
   }
   DCHECK(function->is_compiled());
-  // TODO(v8:11880): avoid roundtrips between cdc and code.
-  return ToCodeT(function->code());
+  return function->code();
 }
 
 RUNTIME_FUNCTION(Runtime_InstallBaselineCode) {
@@ -127,8 +125,7 @@ RUNTIME_FUNCTION(Runtime_FunctionFirstExecution) {
   function->feedback_vector().ClearOptimizationMarker();
   // Return the code to continue execution, we don't care at this point whether
   // this is for lazy compilation or has been eagerly complied.
-  // TODO(v8:11880): avoid roundtrips between cdc and code.
-  return ToCodeT(function->code());
+  return function->code();
 }
 
 RUNTIME_FUNCTION(Runtime_HealOptimizedCodeSlot) {
@@ -141,8 +138,7 @@ RUNTIME_FUNCTION(Runtime_HealOptimizedCodeSlot) {
   function->feedback_vector().EvictOptimizedCodeMarkedForDeoptimization(
       function->raw_feedback_cell(), function->shared(),
       "Runtime_HealOptimizedCodeSlot");
-  // TODO(v8:11880): avoid roundtrips between cdc and code.
-  return ToCodeT(function->code());
+  return function->code();
 }
 
 RUNTIME_FUNCTION(Runtime_InstantiateAsmJs) {
@@ -314,7 +310,7 @@ RUNTIME_FUNCTION(Runtime_CompileForOnStackReplacement) {
   BytecodeOffset osr_offset = DetermineEntryAndDisarmOSRForUnoptimized(frame);
   DCHECK(!osr_offset.IsNone());
 
-  MaybeHandle<Code> maybe_result;
+  MaybeHandle<CodeT> maybe_result;
   Handle<JSFunction> function(frame->function(), isolate);
   if (IsSuitableForOnStackReplacement(isolate, function)) {
     if (FLAG_trace_osr) {
@@ -328,7 +324,7 @@ RUNTIME_FUNCTION(Runtime_CompileForOnStackReplacement) {
   }
 
   // Check whether we ended up with usable optimized code.
-  Handle<Code> result;
+  Handle<CodeT> result;
   if (maybe_result.ToHandle(&result) &&
       CodeKindIsOptimizedJSFunction(result->kind())) {
     DeoptimizationData data =
