@@ -22,6 +22,7 @@
 #include "src/wasm/struct-types.h"
 #include "src/wasm/wasm-constants.h"
 #include "src/wasm/wasm-init-expr.h"
+#include "src/wasm/wasm-limits.h"
 
 namespace v8 {
 
@@ -72,14 +73,110 @@ struct WasmFunction {
   bool declared;
 };
 
+// A representation of a constant expression. The most common expression types
+// are hard-coded, while the rest are represented as a {WireBytesRef}.
+class ConstantExpression {
+ public:
+  enum Kind {
+    kEmpty,
+    kI32Const,
+    kRefNull,
+    kRefFunc,
+    kWireBytesRef,
+    kLastKind = kWireBytesRef
+  };
+
+  union Value {
+    int32_t i32_value;
+    uint32_t index_or_offset;
+    HeapType::Representation repr;
+  };
+
+  ConstantExpression() : bit_field_(KindField::encode(kEmpty)) {}
+
+  static ConstantExpression I32Const(int32_t value) {
+    return ConstantExpression(ValueField::encode(value) |
+                              KindField::encode(kI32Const));
+  }
+  static ConstantExpression RefFunc(uint32_t index) {
+    return ConstantExpression(ValueField::encode(index) |
+                              KindField::encode(kRefFunc));
+  }
+  static ConstantExpression RefNull(HeapType::Representation repr) {
+    return ConstantExpression(ValueField::encode(repr) |
+                              KindField::encode(kRefNull));
+  }
+  static ConstantExpression WireBytes(uint32_t offset, uint32_t length) {
+    return ConstantExpression(OffsetField::encode(offset) |
+                              LengthField::encode(length) |
+                              KindField::encode(kWireBytesRef));
+  }
+
+  Kind kind() const { return KindField::decode(bit_field_); }
+
+  bool is_set() const { return kind() != kEmpty; }
+
+  uint32_t index() const {
+    DCHECK_EQ(kind(), kRefFunc);
+    return ValueField::decode(bit_field_);
+  }
+
+  HeapType::Representation repr() const {
+    DCHECK_EQ(kind(), kRefNull);
+    return static_cast<HeapType::Representation>(
+        ValueField::decode(bit_field_));
+  }
+
+  int32_t i32_value() const {
+    DCHECK_EQ(kind(), kI32Const);
+    return ValueField::decode(bit_field_);
+  }
+
+  WireBytesRef wire_bytes_ref() const {
+    DCHECK_EQ(kind(), kWireBytesRef);
+    return WireBytesRef(OffsetField::decode(bit_field_),
+                        LengthField::decode(bit_field_));
+  }
+
+ private:
+  static constexpr int kValueBits = 32;
+  static constexpr int kLengthBits = 30;
+  static constexpr int kOffsetBits = 30;
+  static constexpr int kKindBits = 3;
+
+  // There are two possible combinations of fields: offset + length + kind if
+  // kind = kWireBytesRef, or value + kind for anything else.
+  using ValueField = base::BitField<uint32_t, 0, kValueBits, uint64_t>;
+  using OffsetField = base::BitField<uint32_t, 0, kOffsetBits, uint64_t>;
+  using LengthField = OffsetField::Next<uint32_t, kLengthBits>;
+  using KindField = LengthField::Next<Kind, kKindBits>;
+
+  // Make sure we reserve enough bits for a {WireBytesRef}'s length and offset.
+  STATIC_ASSERT(kV8MaxWasmModuleSize <= LengthField::kMax + 1);
+  STATIC_ASSERT(kV8MaxWasmModuleSize <= OffsetField::kMax + 1);
+  // Make sure kind fits in kKindBits.
+  STATIC_ASSERT(kLastKind <= KindField::kMax + 1);
+
+  explicit ConstantExpression(uint64_t bit_field) : bit_field_(bit_field) {}
+
+  uint64_t bit_field_;
+};
+
+// We want to keep {ConstantExpression} small to reduce memory usage during
+// compilation/instantiation.
+STATIC_ASSERT(sizeof(ConstantExpression) <= 8);
+
 // Static representation of a wasm global variable.
 struct WasmGlobal {
-  ValueType type;     // type of the global.
-  bool mutability;    // {true} if mutable.
-  WireBytesRef init;  // the initialization expression of the global.
+  ValueType type;           // type of the global.
+  bool mutability;          // {true} if mutable.
+  ConstantExpression init;  // the initialization expression of the global.
   union {
-    uint32_t index;   // index of imported mutable global.
-    uint32_t offset;  // offset into global memory (if not imported & mutable).
+    // Index of imported mutable global.
+    uint32_t index;
+    // Offset into global memory (if not imported & mutable). Expressed in bytes
+    // for value-typed globals, and in tagged words for reference-typed globals.
+    uint32_t offset;
   };
   bool imported;  // true if imported.
   bool exported;  // true if exported.
@@ -100,58 +197,60 @@ struct WasmTag {
 // Static representation of a wasm data segment.
 struct WasmDataSegment {
   // Construct an active segment.
-  explicit WasmDataSegment(WireBytesRef dest_addr)
-      : dest_addr(std::move(dest_addr)), active(true) {}
+  explicit WasmDataSegment(ConstantExpression dest_addr)
+      : dest_addr(dest_addr), active(true) {}
 
   // Construct a passive segment, which has no dest_addr.
   WasmDataSegment() : active(false) {}
 
-  WireBytesRef dest_addr;  // destination memory address of the data.
-  WireBytesRef source;     // start offset in the module bytes.
-  bool active = true;      // true if copied automatically during instantiation.
+  ConstantExpression dest_addr;  // destination memory address of the data.
+  WireBytesRef source;           // start offset in the module bytes.
+  bool active = true;  // true if copied automatically during instantiation.
 };
 
 // Static representation of wasm element segment (table initializer).
 struct WasmElemSegment {
+  enum Status {
+    kStatusActive,      // copied automatically during instantiation.
+    kStatusPassive,     // copied explicitly after instantiation.
+    kStatusDeclarative  // purely declarative and never copied.
+  };
+  enum ElementType { kFunctionIndexElements, kExpressionElements };
+
   // Construct an active segment.
-  WasmElemSegment(ValueType type, uint32_t table_index, WireBytesRef offset)
-      : type(type),
+  WasmElemSegment(ValueType type, uint32_t table_index,
+                  ConstantExpression offset, ElementType element_type)
+      : status(kStatusActive),
+        type(type),
         table_index(table_index),
         offset(std::move(offset)),
-        status(kStatusActive) {}
+        element_type(element_type) {}
 
   // Construct a passive or declarative segment, which has no table index or
   // offset.
-  WasmElemSegment(ValueType type, bool declarative)
-      : type(type),
-        table_index(0),
-        status(declarative ? kStatusDeclarative : kStatusPassive) {}
+  WasmElemSegment(ValueType type, Status status, ElementType element_type)
+      : status(status), type(type), table_index(0), element_type(element_type) {
+    DCHECK_NE(status, kStatusActive);
+  }
 
-  // Construct a passive or declarative segment, which has no table index or
-  // offset.
+  // Default constructor. Constucts an invalid segment.
   WasmElemSegment()
-      : type(kWasmBottom), table_index(0), status(kStatusActive) {}
+      : status(kStatusActive),
+        type(kWasmBottom),
+        table_index(0),
+        element_type(kFunctionIndexElements) {}
 
   WasmElemSegment(const WasmElemSegment&) = delete;
   WasmElemSegment(WasmElemSegment&&) V8_NOEXCEPT = default;
   WasmElemSegment& operator=(const WasmElemSegment&) = delete;
   WasmElemSegment& operator=(WasmElemSegment&&) V8_NOEXCEPT = default;
 
+  Status status;
   ValueType type;
   uint32_t table_index;
-  WireBytesRef offset;
-  struct Entry {
-    enum Kind { kGlobalGetEntry, kRefFuncEntry, kRefNullEntry } kind;
-    uint32_t index;
-    Entry(Kind kind, uint32_t index) : kind(kind), index(index) {}
-    Entry() : kind(kRefNullEntry), index(0) {}
-  };
-  std::vector<Entry> entries;
-  enum Status {
-    kStatusActive,      // copied automatically during instantiation.
-    kStatusPassive,     // copied explicitly after instantiation.
-    kStatusDeclarative  // purely declarative and never copied.
-  } status;
+  ConstantExpression offset;
+  ElementType element_type;
+  std::vector<ConstantExpression> entries;
 };
 
 // Static representation of a wasm import.
@@ -424,7 +523,7 @@ struct WasmTable {
   bool has_maximum_size = false;  // true if there is a maximum size.
   bool imported = false;          // true if imported.
   bool exported = false;          // true if exported.
-  WireBytesRef initial_value;
+  ConstantExpression initial_value;
 };
 
 inline bool is_asmjs_module(const WasmModule* module) {
